# flake8: noqa
from .core import *
from .channels import *
SCHEMA_VERSION = 'v0.17.0'
SCHEMA_URL = 'https://raw.githubusercontent.com/gosling-lang/gosling.js/v0.17.0/src/gosling-schema/gosling.schema.json'
THEMES = {'dark', 'ensembl', 'excel', 'ggplot', 'google', 'igv', 'jbrowse', 'light', 'ucsc', 'warm', 'washu'}
